@@ -1,26 +1,20 @@
 package RDF::LinkedData::ProviderRole;
 
-# Next line is a workaround to problem documented in Error.pm#COMPATIBILITY
-BEGIN { require Moose::Role; Moose::Role->import; *with_role = *with; undef *with };
+use Moose::Role;
 
 use namespace::autoclean;
 
 use RDF::Trine;
-use RDF::Trine::Serializer;
+use RDF::Trine::Serializer::NTriples;
+use RDF::Trine::Serializer::RDFXML;
 use Log::Log4perl qw(:easy);
 use Plack::Response;
-use RDF::Helper::Properties;
-use URI;
-use Module::Load::Conditional qw[can_load];
+use RDF::LinkedData::Predicates;
 
-with_role 'MooseX::Log::Log4perl::Easy';
+with 'MooseX::Log::Log4perl::Easy';
 
 BEGIN {
-  if ($ENV{TEST_VERBOSE}) {
-    Log::Log4perl->easy_init( { level   => $TRACE } );
-  } else {
-    Log::Log4perl->easy_init( { level   => $FATAL } );
-  }
+    Log::Log4perl->easy_init();
 }
 
 
@@ -32,11 +26,11 @@ RDF::LinkedData::ProviderRole - Role providing important functionality for Linke
 
 =head1 VERSION
 
-Version 0.20
+Version 0.05
 
 =cut
 
-our $VERSION = '0.20';
+our $VERSION = '0.05';
 
 
 =head1 SYNOPSIS
@@ -50,7 +44,7 @@ This module is now a L<Moose::Role>. The intention with this role is threefold:
 
 =over
 
-=item * This module may run standalone, in which case the default
+=item *This module may run standalone, in which case the default
 implementation in this role should be sufficient for a working Linked
 Data server. The empty L<RDF::LinkedData> class should provide such a
 default implementation.
@@ -72,26 +66,19 @@ over time.
 
 Consequently, one should not rely in the current API unless you are
 planning to keep track of the development of this module. It is still
-very much in flux, and may change without warning. Just installing and
-using it as a Linked Data server should be OK, however, as its
-important functionality is unlikely to change in non-backwards
-compatible ways.
+very much in flux, and may change without warning.
 
 
 =head1 METHODS
 
 =over
 
-=item C<< new ( store => $store, model => $model, base_uri => $base_uri, 
-                request => $request, endpoint_config => $endpoint_config ) >>
+=item C<< new ( store => $store, model => $model, base => $base, headers_in => $headers_in ) >>
 
 Creates a new handler object based on named parameters, given a store
-config (recommended usage is to pass a hashref of the type that can be
-passed to L<RDF::Trine::Store>->new_with_config, but a simple string
-can also be used) or model and a base URI. Optionally, you may pass a
-L<Plack::Request> object (must be there if you if you plan to call
-C<content>) and an C<endpoint_config> hashref if you want to have a
-SPARQL Endpoint running using the recommended module L<RDF::Endpoint>.
+config string or model and a base URI. Optionally, you may pass a
+Apache request object, and you will need to pass a L<HTTP::Headers>
+object if you plan to call C<content>.
 
 =cut
 
@@ -99,72 +86,28 @@ sub BUILD {
 	my $self = shift;
 
         unless($self->model) {
-	  # First, set the base if none is configured
-	  my $i = 0;
-	  foreach my $source (@{$self->store->{sources}}) {
-	    unless ($source->{base_uri}) {
-	      ${$self->store->{sources}}[$i]->{base_uri} = $self->base_uri;
-	    }
-	    $i++;
-	  }
-	  my $store	= RDF::Trine::Store->new( $self->store );
-	  $self->model(RDF::Trine::Model->new( $store ));
+            my $store	= RDF::Trine::Store->new_with_string( $self->store );
+            $self->model(RDF::Trine::Model->new( $store ));
 	}
 
-        throw Error -text => "No valid RDF::Trine::Model, need either a store config hashref or a model." unless ($self->model);
+        throw Error -text => "No valid RDF::Trine::Model, need either a store config string or a model." unless ($self->model);
 
- 	if ($self->has_endpoint_config) {
-	  use Data::Dumper;
-	  $self->logger->debug('Endpoint config found with parameters: ' . Dumper($self->endpoint_config) );
-
- 	  unless (can_load( modules => { 'RDF::Endpoint' => 0.03 })) {
- 	    throw Error -text => "RDF::Endpoint not installed. Please install or remove its configuration.";
- 	  }
- 	  $self->endpoint(RDF::Endpoint->new($self->model, $self->endpoint_config));
- 	} else {
-	  $self->logger->info('No endpoint config found');
-	}
 }
 
-has store => (is => 'rw', isa => 'HashRef' );
 
-has endpoint_config => (is => 'ro', isa=>'HashRef', predicate => 'has_endpoint_config');
+has store => (is => 'rw', isa => 'Str' );
 
-=item C<< endpoint ( [ $endpoint ] ) >>
 
-Returns the L<RDF::Endpoint> object if it exists or sets it if a
-L<RDF::Endpoint> object is given as parameter. In most cases, it will
-be created for you if you pass a C<endpoint_config> hashref to the
-constructor, so you would most likely not use this method.
+=item C<< headers_in ( [ $headers ] ) >>
+
+Returns the L<HTTP::Headers> object if it exists or sets it if a L<HTTP::Headers> object is given as parameter.
 
 =cut
 
+has headers_in => ( is => 'rw', isa => 'HTTP::Headers', builder => '_build_headers_in');
 
-has endpoint => (is => 'rw', isa => 'RDF::Endpoint', predicate => 'has_endpoint');
-
-
-=item C<< request ( [ $request ] ) >>
-
-Returns the L<Plack::Request> object if it exists or sets it if a L<Plack::Request> object is given as parameter.
-
-=cut
-
-has request => ( is => 'rw', isa => 'Plack::Request');
-
-
-
-=item C<< helper_properties (  ) >>
-
-Returns the L<RDF::Helper::Properties> object if it exists or sets
-it if a L<RDF::Helper::Properties> object is given as parameter.
-
-=cut
-
-has helper_properties => ( is => 'rw', isa => 'RDF::Helper::Properties', lazy => 1, builder => '_build_helper_properties');
-
-sub _build_helper_properties {
-    my $self = shift;
-    return RDF::Helper::Properties->new(model => $self->model);
+sub _build_headers_in {
+    return HTTP::Headers->new() ;
 }
 
 
@@ -182,14 +125,16 @@ has 'type' => (is => 'rw', isa => 'Str', default => '');
 
 =item C<< my_node >>
 
-A node for the requested URI. This node is typically used as the
-subject to find which statements to return as data. This expects to
-get a URI object containing the full URI of the node.
+A node for the requested B<relative> URI. This node is typically used as
+the subject to find which statements to return as data. Note that the
+base URI, set in the constructor or using the C<base> method, is
+prepended to the argument.
 
 =cut
 
 sub my_node {
-    my ($self, $iri) = @_;
+    my ($self, $first) = @_;
+    my $iri	= sprintf( '%s%s', $self->base, $first );
     
     # not happy with this, but it helps for clients that do content sniffing based on filename
     $iri	=~ s/.(nt|rdf|ttl)$//;
@@ -236,19 +181,16 @@ sub content {
     my %output;
     if ($type eq 'data') {
         $self->{_type} = 'data';
-        my ($type, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->request->headers,
-                                                           base => $self->base_uri,
-                                                           namespaces => $self->namespaces);
+        my ($type, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->headers_in);
         my $iter = $model->bounded_description($node);
         $output{content_type} = $type;
         $output{body} = $s->serialize_iterator_to_string ( $iter );
     } else {
         $self->{_type} = 'page';
-        my $preds = $self->helper_properties;
+        my $preds = RDF::LinkedData::Predicates->new($model);
         my $title		= $preds->title( $node );
         my $desc		= $preds->description( $node );
-        my $description	= sprintf('<table about="'. $node->uri_value  .'"' .">%s</table>\n", 
-				  join("\n\t\t", map { sprintf( '<tr><td>%s</td><td>%s</td></tr>', @$_ ) } @$desc) );
+        my $description	= sprintf( "<table>%s</table>\n", join("\n\t\t", map { sprintf( '<tr><td>%s</td><td>%s</td></tr>', @$_ ) } @$desc) );
         $output{content_type} = 'text/html';
         $output{body} =<<"END";
 <?xml version="1.0"?>
@@ -285,59 +227,47 @@ Returns or sets the RDF::Trine::Model object.
 
 has model => (is => 'rw', isa => 'RDF::Trine::Model');
 
-=item C<< base_uri >>
+=item C<< base >>
 
 Returns or sets the base URI for this handler.
 
 =cut
 
-has base_uri => (is => 'rw', isa => 'Str' );
+has base => (is => 'rw', isa => 'Str', default => "http://localhost:3000" );
 
 
 =item C<< response ( $uri ) >>
 
-Will look up what to do with the given URI object and populate the
-response object.
+Will look up what to do with the given URI and populate the response object.
 
 =cut
 
 sub response {
-    my $self = shift;
-    my $uri = URI->new(shift);
+    my ($self, $uri) = @_;
     my $response = Plack::Response->new;
 
-    my $headers_in = $self->request->headers;
-    my $endpoint_path = '/sparql';
-    if ($self->has_endpoint_config && defined($self->endpoint_config->{endpoint_path})) {
-      my $endpoint_path = $self->endpoint_config->{endpoint_path};
-    }
-
-    if($self->has_endpoint && ($uri->path eq $endpoint_path)) {
-      return $self->endpoint->run( $self->request );
-    }
-
     my $type = $self->type;
     $self->type('');
     my $node = $self->my_node($uri);
     $self->logger->info("Try rendering '$type' page for subject node: " . $node->as_string);
     if ($self->count($node) > 0) {
         if ($type) {
-            my $preds = $self->helper_properties;
+            my $preds = RDF::LinkedData::Predicates->new($self->model);
+            
             my $page = $preds->page($node);
             if (($type eq 'page') && ($page ne $node->uri_value . '/page')) {
                 # Then, we have a foaf:page set that we should redirect to
                 $response->status(301);
                 $response->headers->header('Location' => $page);
-                $response->headers->header('Access-Control-Allow-Origin' => '*');
                 return $response;
             }
 
             $self->logger->debug("Will render '$type' page ");
-            if ($headers_in->can('header') && $headers_in->header('Accept')) {
-                $self->logger->debug('Found Accept header: ' . $headers_in->header('Accept'));
+            if ($self->headers_in->can('header') && $self->headers_in->header('Accept')) {
+                $self->logger->debug('Found Accept header: ' . $self->headers_in->header('Accept'));
             } else {
-                $headers_in->header(HTTP::Headers->new('Accept' => 'application/rdf+xml'));
-                $self->logger->warn('Setting Accept header: ' . $headers_in->header('Accept'));
+                $self->headers_in(HTTP::Headers->new('Accept' => 'application/rdf+xml'));
+                $self->logger->warn('Setting Accept header: ' . $self->headers_in->header('Accept'));
             }
             $response->status(200);
             my $content = $self->content($node, $type);
@@ -346,41 +276,29 @@ sub response {
             $response->content($content->{body});
         } else {
             $response->status(303);
-            my ($ct, $s);
+            my $ct;
             eval {
-                ($ct, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $headers_in,
-                                                          base => $self->base_uri,
-                                                          namespaces => $self->namespaces,
-							  extend => {
-								     'text/html'	=> 'html',
-								     'application/xhtml+xml' => 'html'
-								    }
-							  )
-	      };
-            $self->logger->debug("Got $ct content type");
+                ($ct) = RDF::Trine::Serializer->negotiate('request_headers' => $self->headers_in);
+            };
             if ($@) {
-	      $response->status(406);
-	      $response->headers->content_type('text/plain');
-	      $response->body('HTTP 406: No serialization available any specified content type');
-	      return $response;
+                $ct = 'text/html'; # Set it to HTML for now
             }
-            my $newurl = $uri . '/data';
-            unless ($s->isa('RDF::Trine::Serializer')) {
-                my $preds = $self->helper_properties;
+            my $newurl = $self->base . $uri . '/data';
+            unless ($ct =~ /rdf|turtle/) {
+                my $preds = RDF::LinkedData::Predicates->new($self->model);
                 $newurl = $preds->page($node);
             }
             $self->logger->debug('Will do a 303 redirect to ' . $newurl);
             $response->headers->header('Location' => $newurl);
             $response->headers->header('Vary' => join(", ", qw(Accept)));
         }
-	$response->headers->header('Access-Control-Allow-Origin' => '*');
         return $response;
     } else {
         $response->status(404);
         $response->headers->content_type('text/plain');
         $response->body('HTTP 404: Unknown resource');
         return $response;
-      }
+    }
     # We should never get here.
     $response->status(500);
     $response->headers->content_type('text/plain');
@@ -389,16 +307,6 @@ sub response {
 }
 
 
-=item namespaces ( { skos => 'http://www.w3.org/2004/02/skos/core#', dct => 'http://purl.org/dc/terms/' } )
-
-Gets or sets the namespaces that some serializers use for pretty-printing.
-
-=cut
-
-
-
-has 'namespaces' => (is => 'rw', isa => 'HashRef', default => sub { { rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' } } );
-
 
 =back
 
@@ -432,7 +340,7 @@ L<http://lists.perlrdf.org/listinfo/dev>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2010-2011 Kjetil Kjernsmo, Gregory Todd Williams and ABC Startsiden AS.
+Copyright 2010 Gregory Todd Williams and ABC Startsiden AS.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
